/* cookies.c */
#include "cookies.h"
#include <string.h>

#ifdef DEBUG
struct cookie *debug=NULL;
#endif /* DEBUG */

int main(int argc, char *argv[])
{
	if(argc!=2){
		printf("Usage: %s [file]\n", argv[0]);
		exit(0);
	}
	/* We need an interpreter for system(clear) */
	if(!(system(NULL))){
		printf("What?! Not running unix?\n");
		exit(EMISC);
	}
	if((fp=fopen(argv[1], "r"))==NULL){
		printf("Error opening file %s\n", argv[1]);
		exit(EIOFAIL);
	}
	idx=add();        /* first time through */
	show(idx);

	/* main loop */
	while(1){
		char pr;
		if((pr=getchar())!='\n'){
			switch(pr){
				case 'q':
					printf("\nHad enough cookies for one day?\n");
					exit(0);
				case 'l':
					show(--idx);
					getchar();
					break;
#ifdef DEBUG
				case 'd':
					debug=first;
					while(debug!=NULL){
						printf("\nCookie num: %i\n", debug->num);
						printf("       position: %li\n", debug->position);
						printf("\n");
						debug=debug->next;
					}
					exit(0);
#endif /* DEBUG */
			}
		} else {
			idx=add();
			show(idx);
		}
	}
}

/* add - create doubly-linked list
 * 
 * of course, a singly-linked one would do...
 */
int add()
{
	struct cookie *it;
	struct cookie *old;
	int idx;
	if((it=(struct cookie*) malloc(sizeof(struct cookie)))==NULL){
		printf("\nmalloc error!\n");
		exit(EMISC);
	}
	if((it->position=ftell(fp))==-1){
		printf("\nError returned from ftell()! Aborting!\n");
		exit(EIOFAIL);
	}
	if(last==NULL){	/* first item */
		first=it;
		last=it;
		it->prev=NULL;
		it->next=NULL;
		it->num=1;
		return(it->num);
	} else {
		if((idx=reg(it->position))==-1) {
			old=last;
			last=it;
			old->next=it;
			it->prev=old;
			it->next=NULL;
			it->num=old->num + 1;
			return(last->num);
		} else { /* exists */
			return(idx);
		}
	}
}

/* reg - register cookie
 * if file position is not in a cookie in the list,
 * we have a new cookie
 * 
 * returns -1 if cookie is not registered, else
 * number of cookie
 */

int reg(long pos)
{
	struct cookie * iterator=first;
	while(iterator!=NULL){
		if((iterator->position) == pos){
			return(iterator->num);
		}
		iterator=iterator->next;
		
	}
	return(-1);
}
			
/* show - output cookie
 * does fgets-puts until separator (\n%\n) is encountered.
 * moves file position indicator.
 * prints message and returns NULL on out-of-bounds
 */
struct cookie * show(int cookie_num)
{
	struct cookie *it=get_cookie(cookie_num);
	char lbuf[80];
	char * line=&lbuf;
	if(system("clear")){
		printf("clear failed! Aborting...\n");
		exit(EMISC);
	}
	if(it!=NULL){
		if(fseek(fp, it->position, SEEK_SET)!=0){
				printf("\nfseek() error - sanity lost\n");
				exit(EIOFAIL);
		}
	} else { /* cookie is out of bounds */
		printf("This is the first cookie. Press enter to continue, q to quit\n");
		return(NULL);
		/* NOTE: Currently this can only be the first cookie;
		 * or so I think...
		 */
	}
	while(1){
		if(fgets(line, 80, fp) == NULL){
			printf("This is the last cookie. Press l (you may need to press l\n");
			printf("one more time) to go back, q to quit\n");
			/* FIXME: this seems to work, but the list is _not_ 
			 * sane. Can do without a fix for this...
			 */
			return(NULL);
		}
		if(strcmp(line, "%\n") != 0){
			fputs(line, stdout);
		} else { /* separator */
			return(NULL);
		}
	}
}

/* get_cookie - map cookie->num to (struct cookie *) */
struct cookie * get_cookie(int cookie_num)
{
	struct cookie *iterator=first;
	while(iterator!=NULL){
		if(iterator->num==cookie_num){
			return(iterator);
		}
		iterator=iterator->next;
	}
	return(NULL);
}
